@@ -50,10 +50,11 @@ set_error (libcrun_error_t *err)
   else
     {
       cleanup_free char *msg = NULL;
-      asprintf (&msg, "%s: %s", (*err)->msg, strerror ((*err)->status));
-      if (msg == NULL)
-	return NULL;
-      PyErr_SetString (PyExc_RuntimeError, msg);
+      int ret;
+
+      ret = asprintf (&msg, "%s: %s", (*err)->msg, strerror ((*err)->status));
+      if (LIKELY (ret >= 0))
+        PyErr_SetString (PyExc_RuntimeError, msg);
     }
 
   libcrun_error_release (err);
@@ -232,11 +233,11 @@ container_kill (PyObject *self, PyObject *args)
   libcrun_error_t err;
   PyObject *ctx_obj = NULL;
   char *id = NULL;
-  int signal;
+  char *signal;
   libcrun_context_t *ctx;
   int ret;
 
-  if (!PyArg_ParseTuple (args, "Osi", &ctx_obj, &id, &signal))
+  if (!PyArg_ParseTuple (args, "Oss", &ctx_obj, &id, &signal))
     return NULL;
 
   ctx = PyCapsule_GetPointer (ctx_obj, CONTEXT_OBJ_TAG);
@@ -401,11 +402,10 @@ container_update (PyObject *self, PyObject *args)
   if (process == NULL)
     {
       cleanup_free char *msg = NULL;
-      asprintf (&msg, "cannot parse process: %s", parser_err);
-      if (msg == NULL)
-	return NULL;
+      ret = asprintf (&msg, "cannot parse process: %s", parser_err);
+      if (LIKELY (ret >= 0))
+        PyErr_SetString (PyExc_RuntimeError, msg);
       free (parser_err);
-      PyErr_SetString (PyExc_RuntimeError, msg);
       return NULL;
     }
 
